@@ -1,7 +1,7 @@
 // qsamplerChannel.cpp
 //
 /****************************************************************************
-   Copyright (C) 2004-2007, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2004-2014, rncbc aka Rui Nuno Capela. All rights reserved.
    Copyright (C) 2007, 2008 Christian Schoenebeck
 
    This program is free software; you can redistribute it and/or
@@ -32,11 +32,14 @@
 
 #ifdef CONFIG_LIBGIG
 #include "gig.h"
+#ifdef CONFIG_LIBGIG_SF2
+#include "SF.h"
+#endif
 #endif
 
 namespace QSampler {
 
-#define QSAMPLER_INSTRUMENT_MAX 100
+#define QSAMPLER_INSTRUMENT_MAX 128
 
 #define UNICODE_RIGHT_ARROW	QChar(char(0x92), char(0x21))
 
@@ -204,19 +207,19 @@ bool Channel::loadInstrument ( const QString& sInstrumentFile, int iInstrumentNr
 		return false;
 	if (pMainForm->client() == NULL || m_iChannelID < 0)
 		return false;
-	if (!isInstrumentFile(sInstrumentFile))
+	if (!QFileInfo(sInstrumentFile).exists())
 		return false;
-	if (m_iInstrumentStatus == 100 && m_sInstrumentFile == sInstrumentFile && m_iInstrumentNr == iInstrumentNr)
+	if (m_iInstrumentStatus == 100
+		&& m_sInstrumentFile == sInstrumentFile
+		&& m_iInstrumentNr == iInstrumentNr)
 		return true;
 
-	if (
-		::lscp_load_instrument_non_modal(
+	if (::lscp_load_instrument_non_modal(
 			pMainForm->client(),
 			qsamplerUtilities::lscpEscapePath(
 				sInstrumentFile).toUtf8().constData(),
 			iInstrumentNr, m_iChannelID
-		) != LSCP_OK
-	) {
+		) != LSCP_OK) {
 		appendMessagesClient("lscp_load_instrument");
 		return false;
 	}
@@ -757,45 +760,45 @@ bool Channel::channelSetup ( QWidget *pParent )
 
 
 // Redirected messages output methods.
-void Channel::appendMessages( const QString& s ) const
+void Channel::appendMessages ( const QString& sText ) const
 {
 	MainForm *pMainForm = MainForm::getInstance();
 	if (pMainForm)
-		pMainForm->appendMessages(channelName() + ' ' + s);
+		pMainForm->appendMessages(channelName() + ' ' + sText);
 }
 
-void Channel::appendMessagesColor( const QString& s,
-	const QString& c ) const
+void Channel::appendMessagesColor (
+	const QString& sText, const QString& sColor ) const
 {
 	MainForm *pMainForm = MainForm::getInstance();
 	if (pMainForm)
-		pMainForm->appendMessagesColor(channelName() + ' ' + s, c);
+		pMainForm->appendMessagesColor(channelName() + ' ' + sText, sColor);
 }
 
-void Channel::appendMessagesText( const QString& s ) const
+void Channel::appendMessagesText ( const QString& sText ) const
 {
 	MainForm *pMainForm = MainForm::getInstance();
 	if (pMainForm)
-		pMainForm->appendMessagesText(channelName() + ' ' + s);
+		pMainForm->appendMessagesText(channelName() + ' ' + sText);
 }
 
-void Channel::appendMessagesError( const QString& s ) const
+void Channel::appendMessagesError ( const QString& sText ) const
 {
 	MainForm *pMainForm = MainForm::getInstance();
 	if (pMainForm)
-		pMainForm->appendMessagesError(channelName() + "\n\n" + s);
+		pMainForm->appendMessagesError(channelName() + "\n\n" + sText);
 }
 
-void Channel::appendMessagesClient( const QString& s ) const
+void Channel::appendMessagesClient ( const QString& sText ) const
 {
 	MainForm *pMainForm = MainForm::getInstance();
 	if (pMainForm)
-		pMainForm->appendMessagesClient(channelName() + ' ' + s);
+		pMainForm->appendMessagesClient(channelName() + ' ' + sText);
 }
 
 
 // Context menu event handler.
-void Channel::contextMenuEvent( QContextMenuEvent *pEvent )
+void Channel::contextMenuEvent ( QContextMenuEvent *pEvent )
 {
 	MainForm *pMainForm = MainForm::getInstance();
 	if (pMainForm)
@@ -803,8 +806,8 @@ void Channel::contextMenuEvent( QContextMenuEvent *pEvent )
 }
 
 
-// FIXME: Check whether a given file is an instrument file.
-bool Channel::isInstrumentFile ( const QString& sInstrumentFile )
+// FIXME: Check whether a given file is an instrument file (DLS only).
+bool Channel::isDlsInstrumentFile ( const QString& sInstrumentFile )
 {
 	bool bResult = false;
 
@@ -822,23 +825,47 @@ bool Channel::isInstrumentFile ( const QString& sInstrumentFile )
 }
 
 
+// FIXME: Check whether a given file is an instrument file (SF2 only).
+bool Channel::isSf2InstrumentFile ( const QString& sInstrumentFile )
+{
+	bool bResult = false;
+
+	QFile file(sInstrumentFile);
+	if (file.open(QIODevice::ReadOnly)) {
+		char achHeader[12];
+		if (file.read(achHeader, 12) > 0) {
+			bResult = (::memcmp(&achHeader[0], "RIFF", 4) == 0
+					&& ::memcmp(&achHeader[8], "sfbk", 4) == 0);
+		}
+		file.close();
+	}
+
+	return bResult;
+}
+
+
 // Retrieve the instrument list of a instrument file (.gig).
-QStringList Channel::getInstrumentList( const QString& sInstrumentFile,
-	bool bInstrumentNames )
+QStringList Channel::getInstrumentList (
+	const QString& sInstrumentFile,	bool bInstrumentNames )
 {
-	QString sInstrumentName = QFileInfo(sInstrumentFile).fileName();
 	QStringList instlist;
 
-	if (isInstrumentFile(sInstrumentFile)) {
+	const QFileInfo fi(sInstrumentFile);
+	if (!fi.exists()) {
+		instlist.append(noInstrumentName());
+		return instlist;
+	}
+
 #ifdef CONFIG_LIBGIG
-		if (bInstrumentNames) {
+	if (bInstrumentNames) {
+		if (isDlsInstrumentFile(sInstrumentFile)) {
 			RIFF::File *pRiff
 				= new RIFF::File(sInstrumentFile.toUtf8().constData());
-			gig::File  *pGig  = new gig::File(pRiff);
-#if HAVE_LIBGIG_SETAUTOLOAD
+			gig::File *pGig = new gig::File(pRiff);
+		#ifdef CONFIG_LIBGIG_SETAUTOLOAD
 			// prevent sleepy response time on large .gig files
 			pGig->SetAutoLoad(false);
-#endif
+		#endif
 			gig::Instrument *pInstrument = pGig->GetFirstInstrument();
 			while (pInstrument) {
 				instlist.append((pInstrument->pInfo)->Name.c_str());
@@ -847,34 +874,60 @@ QStringList Channel::getInstrumentList( const QString& sInstrumentFile,
 			delete pGig;
 			delete pRiff;
 		}
+	#ifdef CONFIG_LIBGIG_SF2
 		else
+		if (isSf2InstrumentFile(sInstrumentFile)) {
+			RIFF::File *pRiff
+				= new RIFF::File(sInstrumentFile.toUtf8().constData());
+			sf2::File *pSf2 = new sf2::File(pRiff);
+			const int iPresetCount = pSf2->GetPresetCount();
+			for (int iIndex = 0; iIndex < iPresetCount; ++iIndex) {
+				sf2::Preset *pPreset = pSf2->GetPreset(iIndex);
+				if (pPreset) {
+					instlist.append(pPreset->Name.c_str());
+				} else {
+					instlist.append(fi.fileName()
+						+ " [" + QString::number(iIndex) + "]");
+				}
+			}
+			delete pSf2;
+			delete pRiff;
+		}
+	#endif
+	}
 #endif
-		for (int iInstrumentNr = 0; iInstrumentNr < QSAMPLER_INSTRUMENT_MAX; iInstrumentNr++)
-			instlist.append(sInstrumentName + " [" + QString::number(iInstrumentNr) + "]");
+
+	if (instlist.isEmpty()) {
+		for (int iIndex = 0; iIndex < QSAMPLER_INSTRUMENT_MAX; ++iIndex) {
+			instlist.append(fi.fileName()
+				+ " [" + QString::number(iIndex) + "]");
+		}
 	}
-	else instlist.append(noInstrumentName());
 
 	return instlist;
 }
 
 
 // Retrieve the spacific instrument name of a instrument file (.gig), given its index.
-QString Channel::getInstrumentName( const QString& sInstrumentFile,
-	int iInstrumentNr, bool bInstrumentNames )
+QString Channel::getInstrumentName (
+	const QString& sInstrumentFile, int iInstrumentNr, bool bInstrumentNames )
 {
+	const QFileInfo fi(sInstrumentFile);
+	if (!fi.exists())
+		return noInstrumentName();
+
 	QString sInstrumentName;
 
-	if (isInstrumentFile(sInstrumentFile)) {
-		sInstrumentName = QFileInfo(sInstrumentFile).fileName();
 #ifdef CONFIG_LIBGIG
-		if (bInstrumentNames) {
+	if (bInstrumentNames) {
+		if (isDlsInstrumentFile(sInstrumentFile)) {
 			RIFF::File *pRiff
 				= new RIFF::File(sInstrumentFile.toUtf8().constData());
 			gig::File *pGig = new gig::File(pRiff);
-#if HAVE_LIBGIG_SETAUTOLOAD
+		#ifdef CONFIG_LIBGIG_SETAUTOLOAD
 			// prevent sleepy response time on large .gig files
 			pGig->SetAutoLoad(false);
-#endif
+		#endif
 			int iIndex = 0;
 			gig::Instrument *pInstrument = pGig->GetFirstInstrument();
 			while (pInstrument) {
@@ -888,11 +941,26 @@ QString Channel::getInstrumentName( const QString& sInstrumentFile,
 			delete pGig;
 			delete pRiff;
 		}
+	#ifdef CONFIG_LIBGIG_SF2
 		else
+		if (isSf2InstrumentFile(sInstrumentFile)) {
+			RIFF::File *pRiff
+				= new RIFF::File(sInstrumentFile.toUtf8().constData());
+			sf2::File *pSf2 = new sf2::File(pRiff);
+			sf2::Preset *pPreset = pSf2->GetPreset(iInstrumentNr);
+			if (pPreset)
+				sInstrumentName = pPreset->Name.c_str();
+			delete pSf2;
+			delete pRiff;
+		}
+	#endif
+	}
 #endif
+
+	if (sInstrumentName.isEmpty()) {
+		sInstrumentName  = fi.fileName();
 		sInstrumentName += " [" + QString::number(iInstrumentNr) + "]";
 	}
-	else sInstrumentName = noInstrumentName();
 
 	return sInstrumentName;
 }
@@ -1008,7 +1076,12 @@ void ChannelRoutingModel::refresh ( Device *pDevice,
 	m_pDevice = pDevice;
 	m_routing = routing;
 	// inform the outer world (QTableView) that our data changed
+#if QT_VERSION < 0x050000
 	QAbstractTableModel::reset();
+#else
+	QAbstractTableModel::beginResetModel();
+	QAbstractTableModel::endResetModel();
+#endif
 }
 
 
