@@ -1,7 +1,7 @@
 // qsamplerChannelForm.cpp
 //
 /****************************************************************************
-   Copyright (C) 2004-2009, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2004-2012, rncbc aka Rui Nuno Capela. All rights reserved.
    Copyright (C) 2007, 2008 Christian Schoenebeck
 
    This program is free software; you can redistribute it and/or
@@ -64,7 +64,11 @@ ChannelForm::ChannelForm ( QWidget* pParent )
 
 	m_ui.AudioRoutingTable->setModel(&m_routingModel);
 	m_ui.AudioRoutingTable->setItemDelegate(&m_routingDelegate);
+#if QT_VERSION >= 0x050000
+	m_ui.AudioRoutingTable->horizontalHeader()->setSectionResizeMode(0, QHeaderView::Stretch);
+#else
 	m_ui.AudioRoutingTable->horizontalHeader()->setResizeMode(0, QHeaderView::Stretch);
+#endif
 //	m_ui.AudioRoutingTable->verticalHeader()->hide();
 
 	// This goes initially hidden, and will be shown
@@ -107,11 +111,11 @@ ChannelForm::ChannelForm ( QWidget* pParent )
 	QObject::connect(m_ui.AudioDeviceComboBox,
 		SIGNAL(activated(int)),
 		SLOT(selectAudioDevice(int)));
-	QObject::connect(m_ui.OkPushButton,
-		SIGNAL(clicked()),
+	QObject::connect(m_ui.DialogButtonBox,
+		SIGNAL(accepted()),
 		SLOT(accept()));
-	QObject::connect(m_ui.CancelPushButton,
-		SIGNAL(clicked()),
+	QObject::connect(m_ui.DialogButtonBox,
+		SIGNAL(rejected()),
 		SLOT(reject()));
 	QObject::connect(m_ui.MidiDeviceToolButton,
 		SIGNAL(clicked()),
@@ -434,7 +438,7 @@ void ChannelForm::reject (void)
 	bool bReject = true;
 
 	// Check if there's any pending changes...
-	if (m_iDirtyCount > 0 && m_ui.OkPushButton->isEnabled()) {
+	if (m_iDirtyCount > 0) {
 		switch (QMessageBox::warning(this,
 			QSAMPLER_TITLE ": " + tr("Warning"),
 			tr("Some channel settings have been changed.\n\n"
@@ -473,10 +477,21 @@ void ChannelForm::openInstrumentFile (void)
 
 	// FIXME: the instrument file filters should be restricted,
 	// depending on the current engine.
+	QStringList filters;
+	const QString& sEngineName = m_ui.EngineNameComboBox->currentText().toUpper();
+	if (sEngineName.contains("GIG"))
+		filters << tr("GIG Instrument files") + " (*.gig *.dls)";
+	if (sEngineName.contains("SFZ"))
+		filters << tr("SFZ Instrument files") + " (*.sfz)";
+	if (sEngineName.contains("SF2"))
+		filters << tr("SF2 Instrument files") + " (*.sf2)";
+	filters << tr("All files") + " (*.*)";
+	const QString& filter = filters.join(";;");
+
 	QString sInstrumentFile = QFileDialog::getOpenFileName(this,
 		QSAMPLER_TITLE ": " + tr("Instrument files"), // Caption.
-		pOptions->sInstrumentDir,                 // Start here.
-		tr("Instrument files") + " (*.gig *.dls)" // Filter (GIG and DLS files)
+		pOptions->sInstrumentDir, // Start here.
+		filter                    // File filter.
 	);
 
 	if (sInstrumentFile.isEmpty())
@@ -570,7 +585,7 @@ void ChannelForm::selectMidiDriverItem ( const QString& sMidiDriver )
 	m_midiDevices.clear();
 
 	// Populate with the current ones...
-	const QPixmap midiPixmap(":/icons/midi2.png");
+	const QPixmap midiPixmap(":/images/midi2.png");
 	int *piDeviceIDs = Device::getDevices(pMainForm->client(),
 		Device::Midi);
 	for (int i = 0; piDeviceIDs && piDeviceIDs[i] >= 0; i++) {
@@ -685,7 +700,7 @@ void ChannelForm::selectAudioDriverItem ( const QString& sAudioDriver )
 	m_audioDevices.clear();
 
 	// Populate with the current ones...
-	const QPixmap audioPixmap(":/icons/audio2.png");
+	const QPixmap audioPixmap(":/images/audio2.png");
 	int *piDeviceIDs = Device::getDevices(pMainForm->client(),
 		Device::Audio);
 	for (int i = 0; piDeviceIDs && piDeviceIDs[i] >= 0; i++) {
@@ -807,7 +822,8 @@ void ChannelForm::stabilizeForm (void)
 	const QString& sPath = InstrumentFileComboBox->currentText();
 	bValid = bValid && !sPath.isEmpty() && QFileInfo(sPath).exists();
 #endif
-	m_ui.OkPushButton->setEnabled(m_iDirtyCount > 0 && bValid);
+	m_ui.DialogButtonBox->button(
+		QDialogButtonBox::Ok)->setEnabled(m_iDirtyCount > 0 && bValid);
 }
 
 
