@@ -1,8 +1,8 @@
 // qsamplerChannelStrip.h
 //
 /****************************************************************************
-   Copyright (C) 2004-2007, rncbc aka Rui Nuno Capela. All rights reserved.
-   Copyright (C) 2007, 2008 Christian Schoenebeck
+   Copyright (C) 2004-2014, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2007, 2008, 2014 Christian Schoenebeck
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -26,9 +26,10 @@
 #include "ui_qsamplerChannelStrip.h"
 
 #include "qsamplerChannel.h"
-#include <QTimer>
 
 class QDragEnterEvent;
+class QTimer;
+class QMenu;
 
 
 namespace QSampler {
@@ -81,7 +82,8 @@ public slots:
 	bool channelFxEdit();
 	bool channelReset();
 	void volumeChanged(int iVolume);
-	void midiArrived();
+
+	void midiActivityLedOn();
 
 protected:
 
@@ -91,16 +93,24 @@ protected:
 
 protected slots:
 
-	void midiDataCeased();
+	void midiActivityLedOff();
+	void instrumentListPopupItemClicked(QAction* action);
 
 private:
 
 	Ui::qsamplerChannelStrip m_ui;
 
-	Channel* m_pChannel;
+	Channel *m_pChannel;
 	int m_iDirtyChange;
 	int m_iErrorCount;
-	QTimer* pMidiActivityTimer;
+	QMenu* m_instrumentListPopupMenu;
+
+	QTimer  *m_pMidiActivityTimer;
+
+	// MIDI activity pixmap common resources.
+	static int      g_iMidiActivityRefCount;
+	static QPixmap *g_pMidiActivityLedOn;
+	static QPixmap *g_pMidiActivityLedOff;
 
 	// Channel strip activation/selection.
 	static ChannelStrip *g_pSelectedStrip;
