@@ -1,7 +1,7 @@
 // qsamplerInstrumentListForm.cpp
 //
 /****************************************************************************
-   Copyright (C) 2003-2009, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2003-2015, rncbc aka Rui Nuno Capela. All rights reserved.
    Copyright (C) 2007, Christian Schoenebeck
 
    This program is free software; you can redistribute it and/or
@@ -23,6 +23,8 @@
 #include "qsamplerAbout.h"
 #include "qsamplerInstrumentListForm.h"
 
+#include "qsamplerInstrumentList.h"
+
 #include "qsamplerInstrumentForm.h"
 
 #include "qsamplerOptions.h"
@@ -31,6 +33,9 @@
 
 #include <QHeaderView>
 #include <QMessageBox>
+#include <QContextMenuEvent>
+
+#include <QCheckBox>
 
 
 namespace QSampler {
@@ -45,56 +50,31 @@ InstrumentListForm::InstrumentListForm (
 {
 	m_ui.setupUi(this);
 
+	m_pInstrumentListView = new InstrumentListView(this);
+	QMainWindow::setCentralWidget(m_pInstrumentListView);
+
 	// Setup toolbar widgets.
-	m_pMapComboBox = new QComboBox(m_ui.InstrumentToolbar);
+	m_pMapComboBox = new QComboBox(m_ui.instrumentToolbar);
 	m_pMapComboBox->setMinimumWidth(120);
 	m_pMapComboBox->setEnabled(false);
 	m_pMapComboBox->setToolTip(tr("Instrument Map"));
-	m_ui.InstrumentToolbar->addWidget(m_pMapComboBox);
-
-	m_ui.InstrumentToolbar->addSeparator();
-	m_ui.InstrumentToolbar->addAction(m_ui.newInstrumentAction);
-	m_ui.InstrumentToolbar->addAction(m_ui.editInstrumentAction);
-	m_ui.InstrumentToolbar->addAction(m_ui.deleteInstrumentAction);
-	m_ui.InstrumentToolbar->addSeparator();
-	m_ui.InstrumentToolbar->addAction(m_ui.refreshInstrumentsAction);
-
-	int iRowHeight = m_ui.InstrumentTable->fontMetrics().height() + 4;
-	m_ui.InstrumentTable->verticalHeader()->setDefaultSectionSize(iRowHeight);
-
-	m_ui.InstrumentTable->setModel(&m_model);
-	m_ui.InstrumentTable->setItemDelegate(&m_delegate);
-	m_ui.InstrumentTable->verticalHeader()->hide();
-
-	QHeaderView *pHeader = m_ui.InstrumentTable->horizontalHeader();
-	pHeader->setDefaultAlignment(Qt::AlignLeft);
-	pHeader->setMovable(false);
-	pHeader->setStretchLastSection(true);
-	pHeader->resizeSection(0, 120);						// Name
-	m_ui.InstrumentTable->resizeColumnToContents(1);	// Map
-	m_ui.InstrumentTable->resizeColumnToContents(2);	// Bank
-	m_ui.InstrumentTable->resizeColumnToContents(3);	// Prog
-	m_ui.InstrumentTable->resizeColumnToContents(4);	// Engine
-	pHeader->resizeSection(5, 240);						// File
-	m_ui.InstrumentTable->resizeColumnToContents(6);	// Nr
-	pHeader->resizeSection(7, 60);						// Vol
-
-	// Enable custom context menu...
-	m_ui.InstrumentTable->setContextMenuPolicy(Qt::CustomContextMenu);
+	
+	m_ui.instrumentToolbar->addWidget(m_pMapComboBox);
+	m_ui.instrumentToolbar->addSeparator();
+	m_ui.instrumentToolbar->addAction(m_ui.newInstrumentAction);
+	m_ui.instrumentToolbar->addAction(m_ui.editInstrumentAction);
+	m_ui.instrumentToolbar->addAction(m_ui.deleteInstrumentAction);
+	m_ui.instrumentToolbar->addSeparator();
+	m_ui.instrumentToolbar->addAction(m_ui.refreshInstrumentsAction);
 
 	QObject::connect(m_pMapComboBox,
 		SIGNAL(activated(int)),
 		SLOT(activateMap(int)));
-	QObject::connect(
-		m_ui.InstrumentTable,
-		SIGNAL(customContextMenuRequested(const QPoint&)),
-		SLOT(contextMenu(const QPoint&)));
-	QObject::connect(
-		m_ui.InstrumentTable,
-		SIGNAL(pressed(const QModelIndex&)),
+	QObject::connect(m_pInstrumentListView->selectionModel(),
+		SIGNAL(currentRowChanged(const QModelIndex&,const QModelIndex&)),
 		SLOT(stabilizeForm()));
 	QObject::connect(
-		m_ui.InstrumentTable,
+		m_pInstrumentListView,
 		SIGNAL(activated(const QModelIndex&)),
 		SLOT(editInstrument(const QModelIndex&)));
 	QObject::connect(
@@ -114,13 +94,6 @@ InstrumentListForm::InstrumentListForm (
 		SIGNAL(triggered()),
 		SLOT(refreshInstruments()));
 
-	MainForm *pMainForm = MainForm::getInstance();
-	if (pMainForm) {
-		QObject::connect(&m_model,
-			SIGNAL(instrumentsChanged()),
-			pMainForm, SLOT(sessionDirty()));
-	}
-
 	// Things must be stable from the start.
 	stabilizeForm();
 }
@@ -129,6 +102,7 @@ InstrumentListForm::InstrumentListForm (
 InstrumentListForm::~InstrumentListForm (void)
 {
 	delete m_pMapComboBox;
+	delete m_pInstrumentListView;
 }
 
 
@@ -211,57 +185,13 @@ void InstrumentListForm::activateMap ( int iMap )
 	if (iMidiMap >= 0)
 		pOptions->iMidiMap = iMidiMap;
 
-	m_model.setMidiMap(iMidiMap);
-	m_model.refresh();
+	m_pInstrumentListView->setMidiMap(iMidiMap);
+	m_pInstrumentListView->refresh();
 
 	stabilizeForm();
 }
 
 
-void InstrumentListForm::editInstrument (void)
-{
-	editInstrument(m_ui.InstrumentTable->currentIndex());
-}
-
-
-void InstrumentListForm::editInstrument ( const QModelIndex& index )
-{
-	if (!index.isValid() || !index.data(Qt::UserRole).isValid())
-		return;
-
-	Instrument* pInstrument
-		= static_cast<Instrument *> (
-			index.data(Qt::UserRole).value<void *> ());
-
-	if (pInstrument == NULL)
-		return;
-
-	// Save current key values...
-	Instrument oldInstrument(*pInstrument);
-	// Do the edit dance...
-	InstrumentForm form(this);
-	form.setup(pInstrument);
-	if (form.exec()) {
-		// Commit...
-		pInstrument->mapInstrument();
-		// Check whether we changed instrument key...
-		if (oldInstrument.map()  == pInstrument->map()  &&
-			oldInstrument.bank() == pInstrument->bank() &&
-			oldInstrument.prog() == pInstrument->prog()) {
-			// Just update tree item...
-			//pItem->update();
-		} else {
-			// Unmap old instance...
-			oldInstrument.unmapInstrument();
-			// correct the position of the instrument in the model
-			m_model.resort(*pInstrument);
-		}
-		// Notify we've changes...
-		emit m_model.reset();
-	}
-}
-
-
 void InstrumentListForm::newInstrument (void)
 {
 	Instrument instrument;
@@ -273,25 +203,74 @@ void InstrumentListForm::newInstrument (void)
 
 	// Commit...
 	instrument.mapInstrument();
+
 	// add new item to the table model
-	m_model.resort(instrument);
-	// Notify we've changes...
-	//emit model.reset();
-	//FIXME: call above didnt really refresh, so we use this for now ...
-	refreshInstruments();
+	m_pInstrumentListView->addInstrument(
+		instrument.map(),
+		instrument.bank(),
+		instrument.prog());
+
+	stabilizeForm();
+}
+
+
+void InstrumentListForm::editInstrument (void)
+{
+	editInstrument(m_pInstrumentListView->currentIndex());
+}
+
+
+void InstrumentListForm::editInstrument ( const QModelIndex& index )
+{
+	if (!index.isValid())
+		return;
+
+	Instrument *pInstrument
+		= static_cast<Instrument *> (index.internalPointer());
+	if (pInstrument == NULL)
+		return;
+
+	// Save current key values...
+	int iMap  = pInstrument->map();
+	int iBank = pInstrument->bank();
+	int iProg = pInstrument->prog();
+
+	Instrument instrument(iMap, iBank, iProg);
+
+	// Do the edit dance...
+	InstrumentForm form(this);
+	form.setup(pInstrument);
+	if (!form.exec())
+		return;
+	
+	// Commit...
+	pInstrument->mapInstrument();
+
+	// Check whether we changed instrument key...
+	if (pInstrument->map()  == iMap  &&
+		pInstrument->bank() == iBank &&
+		pInstrument->prog() == iProg) {
+		// Just update tree item...
+		m_pInstrumentListView->updateInstrument(pInstrument);
+	} else {
+		// Unmap old instance...
+		instrument.unmapInstrument();
+		// Correct the position of the instrument in the model
+		m_pInstrumentListView->resortInstrument(pInstrument);
+	}
+
+	stabilizeForm();
 }
 
 
 void InstrumentListForm::deleteInstrument (void)
 {
-	const QModelIndex& index = m_ui.InstrumentTable->currentIndex();
-	if (!index.isValid() || !index.data(Qt::UserRole).isValid())
+	const QModelIndex& index = m_pInstrumentListView->currentIndex();
+	if (!index.isValid())
 		return;
 
-	Instrument *pInstrument =
-		static_cast<Instrument*> (
-			index.data(Qt::UserRole).value<void *> ());
-
+	Instrument *pInstrument
+		= static_cast<Instrument *> (index.internalPointer());
 	if (pInstrument == NULL)
 		return;
 
@@ -302,22 +281,39 @@ void InstrumentListForm::deleteInstrument (void)
 	// Prompt user if this is for real...
 	Options *pOptions = pMainForm->options();
 	if (pOptions && pOptions->bConfirmRemove) {
-		if (QMessageBox::warning(this,
-			QSAMPLER_TITLE ": " + tr("Warning"),
-			tr("About to delete instrument map entry:\n\n"
+		const QString& sTitle = QSAMPLER_TITLE ": " + tr("Warning");
+		const QString& sText = tr(
+			"About to delete instrument map entry:\n\n"
 			"%1\n\n"
 			"Are you sure?")
-			.arg(pInstrument->name()),
-			QMessageBox::Ok | QMessageBox::Cancel)
-			== QMessageBox::Cancel)
-			return;
+			.arg(pInstrument->name());
+	 #if 0
+		 if (QMessageBox::warning(this, sTitle, sText,
+			 QMessageBox::Ok | QMessageBox::Cancel) == QMessageBox::Cancel)
+			 return;
+	 #else
+		 QMessageBox mbox(this);
+		 mbox.setIcon(QMessageBox::Warning);
+		 mbox.setWindowTitle(sTitle);
+		 mbox.setText(sText);
+		 mbox.setStandardButtons(QMessageBox::Ok | QMessageBox::Cancel);
+		 QCheckBox cbox(tr("Don't ask this again"));
+		 cbox.setChecked(false);
+		 cbox.blockSignals(true);
+		 mbox.addButton(&cbox, QMessageBox::ActionRole);
+		 if (mbox.exec() == QMessageBox::Cancel)
+			 return;
+		 if (cbox.isChecked())
+			 pOptions->bConfirmRemove = false;
+	 #endif
 	}
 
 	pInstrument->unmapInstrument();
-	// let the instrument vanish from the table model
-	m_model.removeInstrument(*pInstrument);
-	// Notify we've changes...
-	emit m_model.reset();
+
+	// Let the instrument vanish from the table model...
+	m_pInstrumentListView->removeInstrument(pInstrument);
+
+	stabilizeForm();
 }
 
 
@@ -328,21 +324,27 @@ void InstrumentListForm::stabilizeForm (void)
 
 	bool bEnabled = (pMainForm && pMainForm->client());
 	m_ui.newInstrumentAction->setEnabled(bEnabled);
-	const QModelIndex& index = m_ui.InstrumentTable->currentIndex();
+	const QModelIndex& index = m_pInstrumentListView->currentIndex();
 	bEnabled = (bEnabled && index.isValid());
 	m_ui.editInstrumentAction->setEnabled(bEnabled);
 	m_ui.deleteInstrumentAction->setEnabled(bEnabled);
 }
 
 
-// Handle custom context menu here...
-void InstrumentListForm::contextMenu ( const QPoint& pos )
+// Context menu request.
+void InstrumentListForm::contextMenuEvent (
+	QContextMenuEvent *pContextMenuEvent )
 {
-	if (!m_ui.newInstrumentAction->isEnabled())
-		return;
+	QMenu menu(this);
 
-	m_ui.contextMenu->exec(
-		(m_ui.InstrumentTable->viewport())->mapToGlobal(pos));
+	menu.addAction(m_ui.newInstrumentAction);
+	menu.addSeparator();
+	menu.addAction(m_ui.editInstrumentAction);
+	menu.addAction(m_ui.deleteInstrumentAction);
+	menu.addSeparator();
+	menu.addAction(m_ui.refreshInstrumentsAction);
+
+	menu.exec(pContextMenuEvent->globalPos());
 }
 
 
