@@ -1,8 +1,8 @@
 // qsamplerMainForm.h
 //
 /****************************************************************************
-   Copyright (C) 2004-2009, rncbc aka Rui Nuno Capela. All rights reserved.
-   Copyright (C) 2007, 2008 Christian Schoenebeck
+   Copyright (C) 2004-2015, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2007,2008,2015 Christian Schoenebeck
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -28,7 +28,9 @@
 #include <lscp/client.h>
 
 class QProcess;
-class QWorkspace;
+class QMdiArea;
+class QMdiSubWindow;
+class QSocketNotifier;
 class QSpinBox;
 class QSlider;
 class QLabel;
@@ -62,17 +64,17 @@ public:
 
 	QString sessionName(const QString& sFilename);
 
-	void appendMessages(const QString& s);
-	void appendMessagesColor(const QString& s, const QString& c);
-	void appendMessagesText(const QString& s);
-	void appendMessagesError(const QString& s);
-	void appendMessagesClient(const QString& s);
+	void appendMessages(const QString& sText);
+	void appendMessagesColor(const QString& sText, const QString& sColor);
+	void appendMessagesText(const QString& sText);
+	void appendMessagesError(const QString& sText);
+	void appendMessagesClient(const QString& sText);
 
-	ChannelStrip* createChannelStrip(Channel *pChannel);
-	void destroyChannelStrip(ChannelStrip* pChannelStrip);
-	ChannelStrip* activeChannelStrip();
-	ChannelStrip* channelStripAt(int iChannel);
-	ChannelStrip* channelStrip(int iChannelID);
+	ChannelStrip *createChannelStrip(Channel *pChannel);
+	void destroyChannelStrip(ChannelStrip *pChannelStrip);
+	ChannelStrip *activeChannelStrip();
+	ChannelStrip *channelStripAt(int iChannel);
+	ChannelStrip *channelStrip(int iChannelID);
 
 	void contextMenuEvent(QContextMenuEvent *pEvent);
 
@@ -115,12 +117,17 @@ public slots:
 	void sessionDirty();
 	void stabilizeForm();
 
+	void handle_sigusr1();
+
 protected slots:
 
 	void updateRecentFilesMenu();
 
 	// Channel strip activation/selection.
-	void activateStrip(QWidget *pWidget);
+	void activateStrip(QMdiSubWindow *pMdiSubWindow);
+
+	// Channel toolbar orientation change.
+	void channelsToolbarOrientation(Qt::Orientation orientation);
 
 protected:
 
@@ -159,7 +166,8 @@ private:
 
 	Options *m_pOptions;
 	Messages *m_pMessages;
-	QWorkspace *m_pWorkspace;
+	QMdiArea *m_pWorkspace;
+	QSocketNotifier *m_pUsr1Notifier;
 	QString m_sFilename;
 	int m_iUntitled;
 	int m_iDirtyCount;
