@@ -1,7 +1,7 @@
 // qsamplerMessages.cpp
 //
 /****************************************************************************
-   Copyright (C) 2004-2008, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2004-2014, rncbc aka Rui Nuno Capela. All rights reserved.
    Copyright (C) 2007, Christian Schoenebeck
 
    This program is free software; you can redistribute it and/or
@@ -36,6 +36,7 @@
 
 #if !defined(WIN32)
 #include <unistd.h>
+#include <fcntl.h>
 #endif
 
 
@@ -49,6 +50,7 @@ namespace QSampler {
 #define QSAMPLER_MESSAGES_FDREAD    0
 #define QSAMPLER_MESSAGES_FDWRITE   1
 
+
 //-------------------------------------------------------------------------
 // QSampler::Messages - Messages log dockable window.
 //
@@ -111,23 +113,28 @@ Messages::~Messages (void)
 }
 
 
-void Messages::showEvent ( QShowEvent *pEvent )
-{
-	QDockWidget::showEvent(pEvent);
-	emit visibilityChanged(isVisible());
-}
-
-
 // Own stdout/stderr socket notifier slot.
 void Messages::stdoutNotify ( int fd )
 {
 #if !defined(WIN32)
+	// Set non-blocking reads, if not already...
+	const int iFlags = ::fcntl(fd, F_GETFL, 0);
+	int iBlock = ((iFlags & O_NONBLOCK) == 0);
+	if (iBlock)
+		iBlock = ::fcntl(fd, F_SETFL, iFlags | O_NONBLOCK);
+	// Read as much as is available...
+	QString sTemp;
 	char achBuffer[1024];
-	int  cchBuffer = ::read(fd, achBuffer, sizeof(achBuffer) - 1);
-	if (cchBuffer > 0) {
-		achBuffer[cchBuffer] = (char) 0;
-		appendStdoutBuffer(achBuffer);
+	const int cchBuffer = sizeof(achBuffer) - 1;
+	int cchRead = ::read(fd, achBuffer, cchBuffer);
+	while (cchRead > 0) {
+		achBuffer[cchRead] = (char) 0;
+		sTemp.append(achBuffer);
+		cchRead = (iBlock ? 0 : ::read(fd, achBuffer, cchBuffer));
 	}
+	// Needs to be non-empty...
+	if (!sTemp.isEmpty())
+		appendStdoutBuffer(sTemp);
 #endif
 }
 
@@ -137,9 +144,9 @@ void Messages::appendStdoutBuffer ( const QString& s )
 {
 	m_sStdoutBuffer.append(s);
 
-	int iLength = m_sStdoutBuffer.lastIndexOf('\n');
+	const int iLength = m_sStdoutBuffer.lastIndexOf('\n');
 	if (iLength > 0) {
-		QString sTemp = m_sStdoutBuffer.left(iLength);
+		const QString& sTemp = m_sStdoutBuffer.left(iLength);
 		m_sStdoutBuffer.remove(0, iLength + 1);
 		QStringList list = sTemp.split('\n');
 		QStringListIterator iter(list);
@@ -154,7 +161,7 @@ void Messages::flushStdoutBuffer (void)
 {
 	if (!m_sStdoutBuffer.isEmpty()) {
 		appendMessagesText(m_sStdoutBuffer);
-		m_sStdoutBuffer.truncate(0);
+		m_sStdoutBuffer.clear();
 	}
 }
 
@@ -162,7 +169,7 @@ void Messages::flushStdoutBuffer (void)
 // Stdout capture accessors.
 bool Messages::isCaptureEnabled (void)
 {
-	return (bool) (m_pStdoutNotifier != NULL);
+	return (m_pStdoutNotifier != NULL);
 }
 
 void Messages::setCaptureEnabled ( bool bCapture )
@@ -180,10 +187,6 @@ void Messages::setCaptureEnabled ( bool bCapture )
 			::close(m_fdStdout[QSAMPLER_MESSAGES_FDREAD]);
 			m_fdStdout[QSAMPLER_MESSAGES_FDREAD]  = QSAMPLER_MESSAGES_FDNIL;
 		}
-		if (m_fdStdout[QSAMPLER_MESSAGES_FDREAD] != QSAMPLER_MESSAGES_FDNIL) {
-			::close(m_fdStdout[QSAMPLER_MESSAGES_FDREAD]);
-			m_fdStdout[QSAMPLER_MESSAGES_FDREAD]  = QSAMPLER_MESSAGES_FDNIL;
-		}
 	}
 	// Are we going to make up the capture?
 	if (bCapture && m_pStdoutNotifier == NULL && ::pipe(m_fdStdout) == 0) {
@@ -293,7 +296,8 @@ void Messages::appendMessages ( const QString& s )
 
 void Messages::appendMessagesColor ( const QString& s, const QString &c )
 {
-	QString sText = QTime::currentTime().toString("hh:mm:ss.zzz") + ' ' + s;
+	const QString& sText
+		= QTime::currentTime().toString("hh:mm:ss.zzz") + ' ' + s;
 	appendMessagesLine("<font color=\"" + c + "\">" + sText + "</font>");
 	appendMessagesLog(sText);
 }
