@@ -1,7 +1,7 @@
 // qsamplerMessages.h
 //
 /****************************************************************************
-   Copyright (C) 2004-2008, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2004-2014, rncbc aka Rui Nuno Capela. All rights reserved.
    Copyright (C) 2007, Christian Schoenebeck
 
    This program is free software; you can redistribute it and/or
@@ -74,19 +74,12 @@ public:
 	// History reset.
 	void clear();
 
-signals:
-
-	void visibilityChanged(bool bVisible);
-
 protected:
 
 	// Message executives.
 	void appendMessagesLine(const QString& s);
 	void appendMessagesLog(const QString& s);
 
-	// overridden method of QWidget
-	void showEvent(QShowEvent *pEvent);
-
 protected slots:
 
 	// Stdout capture slot.
