@@ -1,8 +1,8 @@
 // qsamplerOptions.h
 //
 /****************************************************************************
-   Copyright (C) 2004-2009, rncbc aka Rui Nuno Capela. All rights reserved.
-   Copyright (C) 2007, Christian Schoenebeck
+   Copyright (C) 2004-2015, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2007,2015 Christian Schoenebeck
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -48,6 +48,10 @@ public:
 	// The settings object accessor.
 	QSettings& settings();
 
+	// explicit I/O methods.
+	void loadOptions();
+	void saveOptions();
+
 	// Command line arguments parser.
 	bool parse_args(const QStringList& args);
 	// Command line usage helper.
@@ -78,6 +82,9 @@ public:
 	bool    bMessagesLimit;
 	int     iMessagesLimitLines;
 	bool    bConfirmRemove;
+	bool    bConfirmReset;
+	bool    bConfirmRestart;
+	bool    bConfirmError;
 	bool    bKeepOnTop;
 	bool    bStdoutCapture;
 	bool    bCompletePath;
@@ -107,8 +114,8 @@ public:
 	QStringList recentFiles;
 
 	// Widget geometry persistence helper prototypes.
-	void saveWidgetGeometry(QWidget *pWidget);
-	void loadWidgetGeometry(QWidget *pWidget);
+	void saveWidgetGeometry(QWidget *pWidget, bool bVisible = false);
+	void loadWidgetGeometry(QWidget *pWidget, bool bVisible = false);
 
 	// Combo box history persistence helper prototypes.
 	void loadComboBoxHistory(QComboBox *pComboBox, int iLimit = 8);
